@@ -18,9 +18,9 @@ from socket import SOCK_STREAM
 import psutil
 from psutil import FREEBSD
 from psutil import LINUX
+from psutil import MACOS
 from psutil import NETBSD
 from psutil import OPENBSD
-from psutil import OSX
 from psutil import POSIX
 from psutil import SUNOS
 from psutil import WINDOWS
@@ -53,6 +53,7 @@ thisproc = psutil.Process()
 class Base(object):
 
     def setUp(self):
+        safe_rmpath(TESTFN)
         if not NETBSD:
             # NetBSD opens a UNIX socket to /var/log/run.
             cons = thisproc.connections(kind='all')
@@ -124,9 +125,9 @@ class Base(object):
         try:
             sys_cons = psutil.net_connections(kind=kind)
         except psutil.AccessDenied:
-            # On OSX, system-wide connections are retrieved by iterating
+            # On MACOS, system-wide connections are retrieved by iterating
             # over all processes
-            if OSX:
+            if MACOS:
                 return
             else:
                 raise
@@ -152,6 +153,7 @@ class TestUnconnectedSockets(Base, unittest.TestCase):
             assert not conn.raddr
             self.assertEqual(conn.status, psutil.CONN_LISTEN)
 
+    @unittest.skipIf(not supports_ipv6(), "IPv6 not supported")
     def test_tcp_v6(self):
         addr = ("::1", get_free_port())
         with closing(bind_socket(AF_INET6, SOCK_STREAM, addr=addr)) as sock:
@@ -166,6 +168,7 @@ class TestUnconnectedSockets(Base, unittest.TestCase):
             assert not conn.raddr
             self.assertEqual(conn.status, psutil.CONN_NONE)
 
+    @unittest.skipIf(not supports_ipv6(), "IPv6 not supported")
     def test_udp_v6(self):
         addr = ("::1", get_free_port())
         with closing(bind_socket(AF_INET6, SOCK_DGRAM, addr=addr)) as sock:
@@ -255,7 +258,7 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
                 server.close()
                 client.close()
 
-    @skip_on_access_denied(only_if=OSX)
+    @skip_on_access_denied(only_if=MACOS)
     def test_combos(self):
         def check_conn(proc, conn, family, type, laddr, raddr, status, kinds):
             all_kinds = ("all", "inet", "inet4", "inet6", "tcp", "tcp4",
@@ -418,7 +421,7 @@ class TestConnectedSocketPairs(Base, unittest.TestCase):
 # =====================================================================
 
 
-class TestSystemWideConnections(unittest.TestCase):
+class TestSystemWideConnections(Base, unittest.TestCase):
     """Tests for net_connections()."""
 
     @skip_on_access_denied()
@@ -453,7 +456,7 @@ class TestSystemWideConnections(unittest.TestCase):
 
     @skip_on_access_denied()
     # See: https://travis-ci.org/giampaolo/psutil/jobs/237566297
-    @unittest.skipIf(OSX and TRAVIS, "unreliable on OSX + TRAVIS")
+    @unittest.skipIf(MACOS and TRAVIS, "unreliable on MACOS + TRAVIS")
     def test_multi_sockets_procs(self):
         # Creates multiple sub processes, each creating different
         # sockets. For each process check that proc.connections()
@@ -471,7 +474,7 @@ class TestSystemWideConnections(unittest.TestCase):
                 import time, os
                 from psutil.tests import create_sockets
                 with create_sockets():
-                    with open('%s', 'w') as f:
+                    with open(r'%s', 'w') as f:
                         f.write(str(os.getpid()))
                     time.sleep(60)
                 """ % fname)
